/*
 *****************************************************
 *
 *  SaVi by Robert Thurman (thurman@geom.umn.edu) and
 *          Patrick Worfolk (worfolk@alum.mit.edu).
 *
 *  Copyright (c) 1997 by The Geometry Center.
 *  This file is part of SaVi.  SaVi is free software;
 *  you can redistribute it and/or modify it only under
 *  the terms given in the file COPYRIGHT which you should
 *  have received along with this file.  SaVi may be
 *  obtained from:
 *  http://savi.sourceforge.net/
 *  http://www.geom.uiuc.edu/locate/SaVi
 *
 *****************************************************
 *
 * main.c
 *
 * $Id: main.c,v 1.9 2004/05/03 22:11:39 lloydwood Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>

#include "utils.h"
#include "constants.h"
#include "globals.h"
#include "sats.h"
#include "savi.h"
#include "tcl_utils.h"
#include "gv_utils.h"

static void decode_cmd_line(int argc, char *argv[]);

int
main(int argc, char *argv[])
{
  char buf[] = "main(update)";

  /* decode command line */
  decode_cmd_line(argc, argv);

  /* initialize geomview if started as a module */
  if (geomview_module) {
    if (!gv_stream_init_proc()) {
      error("could not open pipe to Geomview; savi is now running standalone.");
      geomview_module = FALSE;
    } else if (!gv_init()) {
      error("could not initialise Geomview (main: error in gv_init).");
      geomview_module = FALSE;
    } else {
      GV_BINARY_FORMAT_AVAILABLE = gv_init_proc();
    }
  } else {
    error("For 3D visualization, run from within Geomview.");
  }

  Tcl_FindExecutable(argv[0]);

  interp = Tcl_CreateInterp();

  /* initialize Tcl interface */
  if (!tcl_init(interp)) {
    error_and_exit("could not initialise Tcl (main: error in tcl_init).");
  }

  /* initialize Tk interface */
  if (!tk_init(interp)) {
    error_and_exit("could not initialise Tk (main: error in tk_init).");
  }

  /* load up a single satellite */
  if (!first_filename) {
      satellite_init();
  }
  /* initialise satellites */
  sats_init();

  /* and update interface */
  if (!tcl_script(buf)) {
    error_and_exit("is savi compiled correctly? "
		   "Edit savi startup scripts to set locations.");
  }

  /* loop until exit */
  while (sats_update() && tk_update());

  Tcl_DeleteInterp(interp);

  exit(0);
}

static void
decode_cmd_line(int argc, char *argv[])
{
  int i;
  unsigned int unknown_parameter = FALSE;
  char * script_filename = NULL;
  FILE *script_file = NULL;

  for (i = 1; i < argc; i++) {
    /* argv[0] gives name of binary, not wrapper script */
    if (strcmp(argv[i], "-geomview") == 0) {
      geomview_module = TRUE;
    } else if (strcmp(argv[i], "-large-map") == 0) {
      coverage_large_map = TRUE;
    } else if (strcmp(argv[i], "-dynamic-texture") == 0) {
      geomview_dynamic_texture = TRUE;
    } else if (strcmp(argv[i], "-nologo") == 0) {
      geomview_logo = FALSE;
    } else if (strcmp(argv[i], "-version") == 0) {
      error(Version);
      exit(0);
    } else {
      if (!script_filename) {
	if (NULL == (script_file = fopen(argv[i], "r"))) {
          unknown_parameter = TRUE;
        } else {
	  script_filename = argv[i];
	  fclose(script_file);
	}
      }
    }
  }

  if (script_filename) {
      first_filename = script_filename;
  }

  if (unknown_parameter) {
    error("supported command-line usage is: \n"
	  "savi [-geomview] [-large-map] [-dynamic-texture] [-nologo] [-version] [filename]");
  }

  if (geomview_dynamic_texture) {
    if (!coverage_large_map) {
      error("dynamic texture mapping may require large coverage map (-large-map).");
    }
    if (!geomview_module) {
      error("dynamic texture mapping only under Geomview.");
      geomview_dynamic_texture = FALSE;
    }
  }
}
